/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file MainWindow.cpp
    \brief Implements interface for GUI

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/

#include "MainWindow.h"
#include <QtGui>
#ifdef Q_WS_MAEMO_5
    #include <QMaemo5InformationBox>
#endif


MainWindow::MainWindow(Backbone *backbone, QWidget *parent):
    GUIInterface(parent) {

    this->backbone = backbone;


    initializeUI();

    connectBackbone();
    connectSearchBar();
    connectWordList();
    connectTranslationWidget();
    connectDictManager();
    connectMenu();
    connectBookmarksWidget();

    setExactSearch(false);

    setMinimumSize(750, 400);

    showMaximized();

    searchBarWidget->setFocus();
}

MainWindow::~MainWindow() {
}


void MainWindow::initializeUI() {

    #ifdef Q_WS_MAEMO_5
        setAttribute(Qt::WA_Maemo5StackedWindow);
    #endif


    /*translationWidget is another stacked window, so we don't add it to
      layout, only create it with this widget as parent
      it must be created as first object in main window, otherwise sometimes
      when app starts in maemo, when trying to set stacked window attribute
      it segfaults*/
    translationWidget = new TranslationWidget(this);




    setWindowIcon(QIcon(":/icons/64x64/mdictionary.png"));
    setWindowTitle("mDictionary");

    mainLayout = new QVBoxLayout();
    QWidget* w = new QWidget();
    w->setLayout(mainLayout);
    setCentralWidget(w);

    menuBar = new QMenuBar();
    setMenuBar(menuBar);

    notifyManager = new NotifyManager(this);

    initializeSearchWidgets();

    initializeMenu();
}

void MainWindow::initializeSearchWidgets() {
    searchBarWidget = new SearchBarWidget();

    wordListWidget = new WordListWidget();

    welcomeScreenWidget = new WelcomeScreenWidget();

    #ifdef Q_WS_MAEMO_5
        //At start we set widget as welcome screen widget
        mainLayout->addWidget(welcomeScreenWidget);
        mainLayout->addWidget(searchBarWidget, 0, Qt::AlignBottom);
    #else
        translationWidget->hide();
        //we add word list and welcome screen to splitter
        splitter = new QSplitter(Qt::Horizontal);
        splitter->addWidget(wordListWidget);
        splitter->addWidget(welcomeScreenWidget);
        splitter->setStretchFactor(1, 150);

        mainLayout->addWidget(splitter, 1);
        mainLayout->addWidget(searchBarWidget, 0, Qt::AlignBottom);
    #endif
}

void MainWindow::initializeMenu() {
    initializeMenuWidgets();

#ifdef Q_WS_MAEMO_5
    menuWidget = new MenuWidget(this);

    menuWidget->addSubMenu(tr("Settings"), settingsWidget);
    menuWidget->addSubMenu(tr("Dictionaries"), dictManagerWidget);
    menuWidget->addSubMenu(tr("Bookmarks"), bookmarksWidget);
    menuWidget->addSubMenu(tr("About"), aboutWidget);

    menuBar->addAction(menuWidget);

    connect(menuWidget, SIGNAL(setApplicationMenu(QWidget*)),
            notifyManager, SLOT(setMenu(QWidget*)));
#else
    dictionariesAction = menuBar->addAction(tr("&Dictionaries"));
    connect(dictionariesAction, SIGNAL(triggered()),
            dictManagerWidget, SLOT(show()));

    settingsAction = menuBar->addAction(tr("&Settings"));
    connect(settingsAction, SIGNAL(triggered()),
            settingsWidget, SLOT(show()));

    QMenu* m = menuBar->addMenu(tr("&Bookmarks"));
    bookmarksShowAllAction = new QAction(tr("Show all"), m);

    bookmarksRemoveAllAction = new QAction(tr("Remove all"), m);

    m->addAction(bookmarksShowAllAction);
    m->addAction(bookmarksRemoveAllAction);

    aboutAction = menuBar->addAction(tr("&About"));
    connect(aboutAction, SIGNAL(triggered()),
            aboutWidget, SLOT(show()));

#endif
}

void MainWindow::initializeMenuWidgets() {
    dictManagerWidget = new DictManagerWidget(this);
    dictManagerWidget->hide();

    settingsWidget = new SettingsWidget(this);
    settingsWidget->hide();

    connect(settingsWidget, SIGNAL(notify(Notify::NotifyType,QString)),
            this, SLOT(showNotification(Notify::NotifyType,QString)));

    bookmarksWidget = new BookmarksWidget(this);
    bookmarksWidget->hide();

    aboutWidget = new AboutWidget(this);
    aboutWidget->hide();
}

void MainWindow::closeEvent(QCloseEvent *event) {
    //request to stop all searches and close app
    Q_EMIT quit();
    event->accept();
}

bool MainWindow::isInExactSearch() {
    return _exactSearch;
}

void MainWindow::setExactSearch(bool exact) {
    _exactSearch = exact;
}

void MainWindow::setExactSearchString(QString word) {
    searchString = word;
}

void MainWindow::wordListReady() {
    //gets results from backbone
    QMultiHash<QString, Translation*> backboneResult = backbone->result();
    QHash<QString, QList<Translation*> > searchResult;

    #ifdef Q_WS_MAEMO_5
        hideWelcomeScreen();
    #endif

    //if nothing was found
    if(backboneResult.count() == 0) {
        showNotification(Notify::Info, tr("Can't find any matching words"));

        //show empty list to remove results of old search
        Q_EMIT showWordList(searchResult);
    }
    else {
        //find translations of the same key word
        QMultiHash<QString, Translation*>::iterator i;
        for(i = backboneResult.begin(); i != backboneResult.end(); i++) {
            searchResult[i.key()].push_back(i.value());
        }

        //show search results
        Q_EMIT showWordList(searchResult);


        if(isInExactSearch()) {
            QList<Translation*> exactTranslation;
            if(checkExactSearch(searchResult, exactTranslation)) {
                Q_EMIT searchTranslations(exactTranslation);
            }
            else {
                showNotification(Notify::Info,
                           tr("Can't find exactly matching word"));
            }

            setExactSearch(false);
        }
    }

    wordListWidget->setFocus();
}

bool MainWindow::checkExactSearch(
        QHash<QString, QList<Translation *> > searchResult,
        QList<Translation *> &found) {

    bool foundExactMatch = false;
    QHash<QString, QList<Translation*> >::iterator j;
    for(j = searchResult.begin(); j != searchResult.end(); j++) {
        if(j.key().toLower() == searchString.toLower()
            && !foundExactMatch) {
            found = j.value();
            return true;
        }
    }
    return false;
}

void MainWindow::translationsReady() {
    #ifndef Q_WS_MAEMO_5
        hideWelcomeScreen();
    #endif

    Q_EMIT showTranslation(backbone->xmls());
    wordListWidget->setFocus();
    #ifdef Q_WS_MAEMO_5
        notifyManager->screenChanged();
    #endif
}


void MainWindow::hideWelcomeScreen() {
#ifdef Q_WS_MAEMO_5
    //switch welcome screen with word list
    if(!wordListWidget->isVisible()) {
        mainLayout->removeWidget(welcomeScreenWidget);
        welcomeScreenWidget->deleteLater();

        mainLayout->insertWidget(0, wordListWidget);
    }
#else
    //switch welcome screen with translation widget
    if(!translationWidget->isVisible()) {
        splitter->insertWidget(1,translationWidget);
        splitter->setStretchFactor(1, 150);
        welcomeScreenWidget->deleteLater();
    }
#endif
}

QList<CommonDictInterface*> MainWindow::getPlugins() {
    return backbone->getPlugins();
}

QHash<CommonDictInterface*, bool> MainWindow::getDictionaries() {
    return backbone->getDictionaries();
}


void MainWindow::search(QString word) {
    setExactSearch(false);
    searchBarWidget->search(word);
    #ifdef Q_WS_MAEMO_5
    if(translationWidget->isVisible()) {
            translationWidget->hide();
            update();
        }
    #endif
}

void MainWindow::searchExact(QString word) {
    setExactSearch(true);
    searchBarWidget->search(word);
}

void MainWindow::searchDelay(QString word) {
    searchBarWidget->searchDelay(word);
}





void MainWindow::searchingInterrupted() {
    //make sure to unset exact search mode
    setExactSearch(false);
}

void MainWindow::addToHistory(QList<Translation *> trans) {
    if(trans.count() > 0) {
        backbone->history()->add(trans[0]->key());
    }
}

void MainWindow::historyNext() {
    if(backbone->history()->nextAvailable()) {
        QString next = backbone->history()->next();
        #ifndef Q_WS_MAEMO_5
            setExactSearch(true);
        #endif
        searchDelay(next);
    }
}

void MainWindow::historyPrev() {
    if(backbone->history()->prevAvailable()) {
        #ifndef Q_WS_MAEMO_5
            setExactSearch(true);
        #endif
        QString prev = backbone->history()->previous();
        searchDelay(prev);
    }
}

void MainWindow::disableMenu() {
    #ifdef Q_WS_MAEMO_5
        if(menuBar->actions().contains(menuWidget)) {
              menuBar->removeAction(menuWidget);
        }
    #else
        menuBar->setEnabled(false);
    #endif
}

void MainWindow::enableMenu() {
    #ifdef Q_WS_MAEMO_5
        if(!menuBar->actions().contains(menuWidget)) {
            menuBar->addAction(menuWidget);
        }
    #else
        menuBar->setEnabled(true);
    #endif
}

void MainWindow::showHistory(QPoint p) {

    HistoryListDialog historyDialog(backbone->history()->list(), searchBarWidget);

    #ifndef Q_WS_MAEMO_5
        QPoint newPos = mapFromGlobal(p);
        newPos.setY(searchBarWidget->pos().y() -
                    historyDialog.sizeHint().height());
        newPos.setX(width() - historyDialog.sizeHint().width());

        historyDialog.move(newPos);
    #endif

    if(historyDialog.exec() == QDialog::Accepted) {
        backbone->history()->setCurrentElement(historyDialog.selectedRow());
        searchExact(historyDialog.selectedWord());
    }
}

void MainWindow::setSettings(Settings *s) {
    backbone->setSettings(s);
}

Settings* MainWindow::settings() {
    return backbone->settings();
}


void MainWindow::showNotification(Notify::NotifyType type, QString text) {
    notifyManager->showNotification(type, text);
}

void MainWindow::connectBackbone() {

    connect(this, SIGNAL(searchWordList(QString)),
            this, SIGNAL(setBusy()));

    connect(this, SIGNAL(searchTranslations(QList<Translation*>)),
            this, SIGNAL(setBusy()));

    connect(this, SIGNAL(stopSearching()),
            this, SIGNAL(setIdle()));

    connect(this, SIGNAL(searchWordList(QString)),
            this, SLOT(setExactSearchString(QString)));

    connect(this, SIGNAL(searchTranslations(QList<Translation*>)),
            this, SLOT(addToHistory(QList<Translation*>)));



    connect(this, SIGNAL(quit()),
            backbone, SLOT(quit()));

    connect(this, SIGNAL(searchWordList(QString)),
            backbone, SLOT(search(QString)));

    connect(this, SIGNAL(searchTranslations(QList<Translation*>)),
            backbone, SLOT(searchXml(QList<Translation*>)));

    connect(this, SIGNAL(stopSearching()),
            backbone, SLOT(stopSearching()));

    connect(this, SIGNAL(stopSearching()),
            this, SLOT(searchingInterrupted()));

    connect(this, SIGNAL(addNewDictionary(CommonDictInterface*)),
            backbone, SLOT(addDictionary(CommonDictInterface*)));

    connect(this, SIGNAL(removeDictionary(CommonDictInterface*)),
            backbone, SLOT(removeDictionary(CommonDictInterface*)));

    connect(this, SIGNAL(selectedDictionaries(QList<CommonDictInterface*>)),
            backbone, SLOT(selectedDictionaries(QList<CommonDictInterface*>)));


    connect(backbone, SIGNAL(ready()),
            this, SIGNAL(setIdle()));

    connect(backbone, SIGNAL(xmlReady()),
            this, SIGNAL(setIdle()));


    connect(backbone, SIGNAL(ready()),
            this, SLOT(wordListReady()));

    connect(backbone, SIGNAL(xmlReady()),
            this, SLOT(translationsReady()));

    connect(backbone, SIGNAL(searchCanceled()),
            this, SIGNAL(setIdle()));

    connect(backbone, SIGNAL(notify(Notify::NotifyType,QString)),
            this, SLOT(showNotification(Notify::NotifyType,QString)));

    connect(backbone, SIGNAL(closeOk()),
            this, SLOT(close()));


    //connect(wordListWidget, SIGNAL(addBookmark(QList<Translation*>)),
    //       this, SIGNAL(setBusy()));

    //connect(backbone, SIGNAL(bookmarkReady()),
    //        this, SIGNAL(setIdle()));
}

void MainWindow::connectSearchBar() {
    connect(searchBarWidget, SIGNAL(searchForTranslations(QString)),
            this, SIGNAL(searchWordList(QString)));

    connect(searchBarWidget, SIGNAL(stopSearching()),
            this, SIGNAL(stopSearching()));

    connect(this, SIGNAL(setBusy()),
            searchBarWidget, SLOT(setBusy()));

    connect(this, SIGNAL(setIdle()),
            searchBarWidget, SLOT(setIdle()));

    connect(searchBarWidget, SIGNAL(historyNext()),
            this, SLOT(historyNext()));

    connect(searchBarWidget, SIGNAL(historyPrev()),
            this, SLOT(historyPrev()));

    connect(searchBarWidget, SIGNAL(historyShow(QPoint)),
            this, SLOT(showHistory(QPoint)));

    connect(searchBarWidget, SIGNAL(refreshHistoryButtons()),
            backbone->history(), SLOT(refreshStatus()));

    connect(backbone->history(), SIGNAL(historyChanged(bool,bool,bool)),
            searchBarWidget, SLOT(updateHistoryButtons(bool,bool,bool)));
}

void MainWindow::connectWordList() {
    connect(this,
            SIGNAL(showWordList(QHash<QString, QList<Translation*> >)),
            wordListWidget,
            SLOT(showSearchResults(QHash<QString,QList<Translation*> >)));

    connect(wordListWidget, SIGNAL(showTranslation(QList<Translation*>)),
            this, SIGNAL(searchTranslations(QList<Translation*>)));




    connect(this, SIGNAL(setBusy()),
            wordListWidget, SLOT(lockList()));

    connect(this, SIGNAL(setIdle()),
            wordListWidget, SLOT(unlockList()));

    connect(wordListWidget, SIGNAL(addBookmark(QList<Translation*>)),
            backbone, SLOT(addBookmark(QList<Translation*>)));

    connect(wordListWidget, SIGNAL(removeBookmark(QList<Translation*>)),
            backbone, SLOT(removeBookmark(QList<Translation*>)));
}

void MainWindow::connectTranslationWidget() {
    connect(this, SIGNAL(showTranslation(QStringList)),
            translationWidget, SLOT(show(QStringList)));

     #ifdef Q_WS_MAEMO_5
        connect(translationWidget, SIGNAL(search(QString)),
                this, SLOT(search(QString)));

        connect(translationWidget, SIGNAL(notify(Notify::NotifyType, QString)),
                this, SLOT(showNotification(Notify::NotifyType,QString)));
    #else
        connect(translationWidget, SIGNAL(search(QString)),
                this, SLOT(searchExact(QString)));
    #endif

}

void MainWindow::connectDictManager() {
    connect(dictManagerWidget, SIGNAL(addDictionary(CommonDictInterface*)),
            this, SIGNAL(addNewDictionary(CommonDictInterface*)));

    connect(dictManagerWidget, SIGNAL(removeDictionary(CommonDictInterface*)),
            this, SIGNAL(removeDictionary(CommonDictInterface*)));

    connect(dictManagerWidget,
            SIGNAL(selectedDictionaries(QList<CommonDictInterface*>)),
            this, SIGNAL(selectedDictionaries(QList<CommonDictInterface*>)));
}

void MainWindow::connectMenu() {
    connect(this, SIGNAL(setBusy()),
            this, SLOT(disableMenu()));

    connect(this, SIGNAL(setIdle()),
            this, SLOT(enableMenu()));
}


void MainWindow::connectBookmarksWidget() {
    #ifdef Q_WS_MAEMO_5
        //after removing bookmarks we search for them once again to clear the words list
        connect(bookmarksWidget, SIGNAL(removeAllBookmarks()),
                this, SLOT(removeBookmarks()));


        connect(bookmarksWidget, SIGNAL(showAllBookmarks()),
                menuWidget, SLOT(hideMenu()));

        connect(bookmarksWidget, SIGNAL(showAllBookmarks()),
                backbone, SLOT(fetchBookmarks()));


    #else
        connect(bookmarksRemoveAllAction, SIGNAL(triggered()),
                this, SLOT(removeBookmarks()));
        connect(bookmarksShowAllAction, SIGNAL(triggered()),
                backbone, SLOT(fetchBookmarks()));

    #endif
}


void MainWindow::removeBookmarks() {
    QWidget* par;
    #ifdef Q_WS_MAEMO_5
        par = bookmarksWidget;
    #else
        par = this;
    #endif
    if(QMessageBox::question(par, tr("Delete all bookmarks"),
             tr("Do you want to delete all bookmarks? (This action cannot be revoked, and will clear current word list)"),
             QMessageBox::Yes, QMessageBox::Cancel) == QMessageBox::Yes) {
        backbone->removeAllBookmarks();
        ((WordListWidget*)wordListWidget)->clear();
    }


}
