/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file SearchBarWidget.cpp
    \brief Displays search bar

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/


#include "SearchBarWidget.h"
#include <QDebug>
#include "../../include/DictDialog.h"
#include "HistoryListDialog.h"


SearchBarWidget::SearchBarWidget(QWidget *parent) : QWidget(parent) {

    initializeUI();


    busy = false;

    connect(searchPushButton, SIGNAL(clicked()),
            this, SLOT(searchPushButtonClicked()));

    connect(searchWordLineEdit, SIGNAL(returnPressed()),
            this, SLOT(searchPushButtonClicked()));

    connect(historyNextToolButton, SIGNAL(clicked()),
            this, SIGNAL(historyNext()));

    connect(historyPrevToolButton, SIGNAL(clicked()),
            this, SIGNAL(historyPrev()));

    connect(historyShowToolButton, SIGNAL(clicked()),
            this, SLOT(showHistoryButtonClicked()));

    connect(clearSearchWordToolButton, SIGNAL(clicked()),
            this, SLOT(clearSearchWordToolButtonClicked()));


    connect(&delayTimer, SIGNAL(timeout()),
            this, SLOT(delaySearchTimeout()));


    searchWordLineEdit->setFocus();

    historyPrevToolButton->setEnabled(false);
    historyNextToolButton->setEnabled(false);
    historyShowToolButton->setEnabled(false);

    setEnabled(true);
}

SearchBarWidget::~SearchBarWidget() {

}

QIcon SearchBarWidget::generateIcon(QIcon original, qreal rotation) {
    QPixmap p = original.pixmap(64);


    if(rotation != 0) {
        QMatrix m;
        m.rotate(rotation);

        p = p.transformed(m);
    }

    QIcon newIcon;
    newIcon.addPixmap(p);


    #ifdef Q_WS_MAEMO_5
        QImage img = p.toImage();

        for(int i=0; i < img.width(); i++) {
            for(int j=0; j < img.height(); j++) {
                QColor c = img.pixel(i,j);
                if(c != QColor(0,0,0,255)) {
                    c.setRed(c.red()/2);
                    c.setGreen(c.green()/2);
                    c.setBlue(c.blue()/2);
                    img.setPixel(i, j, c.rgb());
                }
            }
        }
        p = p.fromImage(img);

        newIcon.addPixmap(p, QIcon::Disabled, QIcon::Off);
    #endif

    return newIcon;
}


void SearchBarWidget::setFocus() {
    searchWordLineEdit->setFocus();
}

void SearchBarWidget::initializeUI() {

    #ifdef Q_WS_MAEMO_5
        setMaximumHeight(150);
    #else
        setMaximumHeight(100);
    #endif


    horizontalLayout = new QHBoxLayout;
    verticalLayout = new QVBoxLayout;


    searchPushButton = new QPushButton(tr("Search"));
    searchPushButton->setMinimumWidth(125);


    searchWordLineEdit = new QLineEdit;
    searchWordLineEdit->setMinimumWidth(250);



    completerModel = new QStringListModel(this);


    lineEditCompleter = new QCompleter(searchWordLineEdit);
    lineEditCompleter->setModel(completerModel);
    lineEditCompleter->setCaseSensitivity(Qt::CaseInsensitive);
    lineEditCompleter->setCompletionMode(QCompleter::InlineCompletion);
    searchWordLineEdit->setCompleter(lineEditCompleter);


    #ifndef Q_WS_MAEMO_5
        searchWordLineEdit->setMinimumHeight(
                searchWordLineEdit->sizeHint().height()*3/2);
    #endif


    //create layout for lineEdit to have clear button on it
    QHBoxLayout* lineEditLayout = new QHBoxLayout;
    searchWordLineEdit->setLayout(lineEditLayout);


    clearSearchWordToolButton = new QToolButton;
    #ifdef Q_WS_MAEMO_5
        clearSearchWordToolButton->setIcon(QIcon::fromTheme("general_stop"));
        clearSearchWordToolButton->setMaximumSize(
                clearSearchWordToolButton->sizeHint().height()/2,
                clearSearchWordToolButton->sizeHint().height()/2);
        lineEditLayout->setContentsMargins(0,0,15,0);
    #else
        clearSearchWordToolButton->setIcon(QIcon::fromTheme("edit-clear"));
        clearSearchWordToolButton->setMinimumSize(
                searchWordLineEdit->sizeHint().height()*1.2,
                searchWordLineEdit->sizeHint().height()*1.2);
        lineEditLayout->setContentsMargins(0,0,5,0);
    #endif


    historyNextToolButton = new QToolButton;
    #ifdef Q_WS_MAEMO_5
        historyNextToolButton->setIcon(
                generateIcon(QIcon::fromTheme("general_forward")));
    #else
        historyNextToolButton->setIcon(
                generateIcon(QIcon::fromTheme("go-next")));
    #endif



    historyPrevToolButton = new QToolButton;
    #ifdef Q_WS_MAEMO_5
        historyPrevToolButton->setIcon(
                generateIcon(QIcon::fromTheme("general_back")));
    #else
        historyPrevToolButton->setIcon(
                generateIcon(QIcon::fromTheme("go-previous")));
    #endif



    historyShowToolButton = new QToolButton;
    #ifdef Q_WS_MAEMO_5
        historyShowToolButton->setIcon(
                generateIcon(QIcon::fromTheme("general_back"), 90));
    #else
        historyShowToolButton->setIcon(
                generateIcon(QIcon::fromTheme("go-up")));
    #endif

    searchingProgressBar = new QProgressBar;
    //progress bar has minimum and maximum values set to 0, which will effect
    //with "I'm alive" bar
    searchingProgressBar->setMinimum(0);
    searchingProgressBar->setMaximum(0);
    #ifdef Q_WS_MAEMO_5
        searchingProgressBar->setMaximumHeight(50);
    #endif
    searchingProgressBar->hide();


    setLayout(verticalLayout);

    verticalLayout->addWidget(searchingProgressBar);

    //adding widgets to layout
    horizontalLayout->addWidget(searchWordLineEdit);
    horizontalLayout->addWidget(searchPushButton);
    horizontalLayout->addWidget(historyPrevToolButton);
    horizontalLayout->addWidget(historyShowToolButton);
    horizontalLayout->addWidget(historyNextToolButton);

    //adding clear toolButton to textEdit with right alignment
    lineEditLayout->addWidget(clearSearchWordToolButton, 0, Qt::AlignRight);


    verticalLayout->addLayout(horizontalLayout);
}


void SearchBarWidget::searchPushButtonClicked() {
    if(busy) {
        Q_EMIT stopSearching();
    }
    else {
        search(searchWordLineEdit->text());
    }
}


void SearchBarWidget::search(QString word) {
    if(!busy && !word.isEmpty()) {
        completerModel->insertRow(completerModel->rowCount());
        QModelIndex index =
                completerModel->index(completerModel->rowCount() -1);

        completerModel->setData(index, word);


        searchWordLineEdit->setText(word);
        Q_EMIT searchForTranslations(word);
    }
}

void SearchBarWidget::searchDelay(QString word) {
    if(!busy && !word.isEmpty()) {
        searchWordLineEdit->setText(word);


        if(delayTimer.isActive()) {
            delayTimer.stop();
        }

        delayString = word;
        delayTimer.start(500);
    }
}

void SearchBarWidget::delaySearchTimeout() {
    delayTimer.stop();
    if(!busy) {
        Q_EMIT searchForTranslations(delayString);
    }
}

void SearchBarWidget::setEnabled(bool enabled) {
    searchWordLineEdit->setEnabled(enabled);

    if(!enabled) {
        historyPrevToolButton->setEnabled(false);
        historyNextToolButton->setEnabled(false);
        historyShowToolButton->setEnabled(false);
    }
}

void SearchBarWidget::setBusy() {
    if(busy) return;
    searchingProgressBar->show();
    searchPushButton->setText(tr("Stop"));
    setEnabled(false);
    busy = true;
}

void SearchBarWidget::setIdle() {
    if(!busy) return;
    searchingProgressBar->hide();
    searchPushButton->setText(tr("Search"));
    setEnabled(true);
    busy = false;
    Q_EMIT refreshHistoryButtons();
}


void SearchBarWidget::clearSearchWordToolButtonClicked() {
    searchWordLineEdit->clear();
}



void SearchBarWidget::updateHistoryButtons(bool prev, bool next, bool list) {
    if(!busy) {
        historyPrevToolButton->setEnabled(prev);
        historyNextToolButton->setEnabled(next);
        historyShowToolButton->setEnabled(list);
    }
}

void SearchBarWidget::showHistoryButtonClicked() {
    #ifdef Q_WS_MAEMO_5
        emit historyShow();
    #else
        QPoint p = historyShowToolButton->pos();
        p.setY(p.y());
        emit historyShow(mapToGlobal(p));
    #endif
}
