/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file GooglePlugin.cpp
    \brief Implementation of google plugin's main class.

    \author Jakub Jaszczynski <j.j.jaszczynski@gmail.com>
*/

#include "GooglePlugin.h"
#include <QDebug>
#include "GoogleDictDialog.h"

QMap<QString, QString> GooglePlugin::languages;
bool GooglePlugin::noNetworkErrorShowed = false;


GooglePlugin::GooglePlugin(QObject *parent): CommonDictInterface(parent),
                    _name(""),_infoNote("") {
    _settings = new Settings();
    _settings->setValue("lang_to","");
    _settings->setValue("lang_from","");
    _settings->setValue("type","google");
    _settings->setValue("connection_accepted","true");
    _dictDialog = new GoogleDictDialog(this,this);
    _icon = QIcon("/usr/share/mdictionary/google.png");

    stopped = false;
    initLanguages();

    http = new QHttp();
    connect(http, SIGNAL(done(bool)), this, SLOT(done()));
    threa=new QThread();
    http->moveToThread(threa);
    threa->start();
}

void GooglePlugin::retranslate() {
    QString locale = QLocale::system().name();
    QTranslator *translator = new QTranslator(this);
    if(!translator->load(":/google/translations/" + locale)) {
        translator->load(":/google/translations/en_US");
    }
    QCoreApplication::installTranslator(translator);
}


GooglePlugin::~GooglePlugin() {
    delete _settings;
    delete _dictDialog;
}


QString GooglePlugin::langFrom() const {
    return languages.key(_settings->value("lang_from"));
}


QString GooglePlugin::langTo() const {
    return languages.key(_settings->value("lang_to"));
}


QString GooglePlugin::name() const {
    return QString("dictionary");
}


QString GooglePlugin::type() const {
    return QString("google");
}


QString GooglePlugin::infoNote() const {
    return _infoNote;
}


void GooglePlugin::setLangTo(QString langTo){
    _settings->setValue("lang_to",langTo);
}


void GooglePlugin::setLangFrom(QString langFrom){
    _settings->setValue("lang_from",langFrom);;
}


DictDialog* GooglePlugin::dictDialog() {
    return _dictDialog;
}


bool GooglePlugin::isAvailable() const {
    return isConnectionAccept();
}


void GooglePlugin::setConnectionAccept(QString connectionAcepted) {
    if(connectionAcepted=="true")
        _settings->setValue("connection_accepted","true");
    else
        _settings->setValue("connection_accepted","false");
}


bool GooglePlugin::isConnectionAccept() const {
    if(_settings->value("connection_accepted")=="true")
        return true;
    else
        return false;
}


Settings* GooglePlugin::settings() {
    return _settings;
}


void GooglePlugin::setSettings(const Settings* settings) {
    if(settings) {
        QStringList list = settings->keys();
        foreach(QString key, list)
            _settings->setValue(key, settings->value(key));
        getDictionaryInfo();
        Q_EMIT settingsChanged();
    }
}


QIcon* GooglePlugin::icon() {
    return &_icon;
}


CommonDictInterface* GooglePlugin::getNew(const Settings* settings) const {
    GooglePlugin *plugin = new GooglePlugin();
    if(settings)
        plugin->setSettings(settings);
    return plugin;
}


QString GooglePlugin::search(QString) {
    qDebug() << "function is not used in google plugin";
    return QString("");
}


Translation* GooglePlugin::getTranslationFor(QString key) {
    return (new TranslationGoogle(key,"",_infoNote,this));
}


QList<Translation*> GooglePlugin::searchWordList(QString word, int ) {
    QList<Translation*> translations;
    if(isAvailable()) {
        QString error("");
        char wrongChar[]={"*?&<>"};
        for(unsigned int a=0;a<sizeof(wrongChar);a++)    /*remove wrong char*/
            word.replace(wrongChar[a],"");

        QString url=QString("/translate_a/t?client=t&sl=%1&tl=%2").arg(
                                              _settings->value("lang_from"),
                                              _settings->value("lang_to"));
        QHttpRequestHeader head = QHttpRequestHeader("POST", url, 1,1);
        head.setValue("Host","www.google.pl");
        head.setValue("User-Agent", "Mozilla/5.0");
        head.setValue("Accept-Encoding", "deflate");
        head.setContentLength(word.length());
        head.setValue("Connection", "Close");

        QByteArray data("text=");
        data.append(word.toUtf8());
        http->setHost("www.google.pl");

        wait=true;      /* bool - change in slot done (initiate by http) */
        stopped=false;  /* bool - change in slot stop (initiate in gui)  */
        int lastState=0;
        http->request(head, data);
        while(wait && (error=="" || error=="Unknown error") && !stopped){
            error=http->errorString();
            if(lastState!=http->state()) {
                lastState=http->state();
                if(lastState==0)
                    done();
            }
        }
        if(error!="" && error!="Unknown error") {
            if(!noNetworkErrorShowed) {
                noNetworkErrorShowed = true;
                Q_EMIT notify(Notify::Warning,
                        tr("GooglePlugin: %1").arg(error));
            }
        }

        QString text = QString::fromUtf8(http->readAll());
        text=jsonParse(text);
        if(text!="") {
            text="<key>" + word + "</key>" + "<t>" + text + "</t>";
            translations<<(new TranslationGoogle(word,text,_infoNote,this));
        }
    }
    return translations;
}


QString GooglePlugin::jsonParse(QString result) {
    int pos=0,pos2=0,index=0,size=0;
    QString returnLang;
    QString translation;
    QString original;
    QList<QString> partOfSpeach;
    QList<QList<QString>* > words;
    QStringList list1 = result.split("\"");

    size=(list1.size()-1)/2;
    if(size<=2)
        return QString(""); // wrong format of data

    translation=list1.at(index*2+1);
    index++;
    original=list1.at(index*2+1);
    pos=result.indexOf("]");
    index++;
    while(result.at(pos+1)==QChar(',')) {
        index++;
        translation+=list1.at(index*2+1);
        index++;
        original=list1.at(index*2+1);
        pos=result.indexOf("]",pos+1);
        index++;
    }

    pos=result.indexOf("]",pos+1);
    pos++;
    index++;
    if(result.at(pos+1)==QChar(','))
        returnLang=list1.at(index*2+1); /*return when translate sentence*/

    while(result.indexOf("[",pos+1)!=-1){
        partOfSpeach.append(list1.at(index*2+1));
        pos2=result.indexOf("]",pos+1);
        pos=result.indexOf("\"",pos+1);
        pos=result.indexOf("\"",pos+1);
        pos=result.indexOf("\"",pos+1);
        pos=result.indexOf("\"",pos+1);
        QList<QString> *list=new QList<QString>;
        while(pos2>pos && pos2!=-1 && pos!= -1) {
            index++;
            if(size==index)
                return QString("");
            list->append(list1.at(index*2+1));
            pos=result.indexOf("\"",pos+1);
            pos=result.indexOf("\"",pos+1);
        }
        words.append(list);
        index++;
        if(size==index)
            return QString("");
        pos=pos2+2;
    }

    if(words.size()!=partOfSpeach.size()) {
        qDebug()<<"Error in Google Plugin (JsonParse)";
        Q_EMIT notify(Notify::Warning,
                tr("GooglePlugin: can't parse Json"));
        return QString("");
    }
    returnLang=list1.at(index*2+1);
    if(partOfSpeach.size()==0){
        if(translation.toLower()==original.toLower())
            return QString(""); // word doesn't exist";
        else
            return "\""+translation+"\"";
    }
    result=translation;
    for(int i=0; i<words.size();i++) {
        result+="<br/><pos>"+partOfSpeach.at(i)+"</pos>: ";
        for(int j=0; j<words.at(i)->size();j++)
            result+=words.at(i)->at(j)+", ";
        result.remove(result.size()-2,2);
    }
    return result;
}


void GooglePlugin::done() {
    wait=false;
}


void GooglePlugin::stop() {
    stopped=true;
}


void GooglePlugin::initLanguages() {
    if(languages.count() != 0) return;

    languages["Afrikaans"] = "af";
    languages["Albanian"] = "sq";
    languages["Arabic"] = "ar";
    languages["Armenian"] = "hy";
    languages["Azerbaijani"] = "az";
    languages["Basque"] = "eu";
    languages["Belarusian"] = "be";
    languages["Bulgarian"] = "bg";
    languages["Catalan"] = "ca";
    languages["Chinese"] = "zh";
    languages["Croatian"] = "hr";
    languages["Czech"] = "cs";
    languages["Danish"] = "da";
    languages["Dutch"] = "nl";
    languages["English"] = "en";
    languages["Estonian"] = "et";
    languages["Filipino"] = "tl";
    languages["Finnish"] = "fi";
    languages["French"] = "fr";
    languages["Galician"] = "gl";
    languages["Georgian"] = "ka";
    languages["German"] = "de";
    languages["Greek"] = "el";
    languages["Haitian_creole"] = "ht";
    languages["Hebrew"] = "iw";
    languages["Hindi"] = "hi";
    languages["Hungarian"] = "hu";
    languages["Icelandic"] = "is";
    languages["Indonesian"] = "id";
    languages["Irish"] = "ga";
    languages["Italian"] = "it";
    languages["Japanese"] = "ja";
    languages["Korean"] = "ko";
    languages["Latvian"] = "lv";
    languages["Lithuanian"] = "lt";
    languages["Macedonian"] = "mk";
    languages["Malay"] = "ms";
    languages["Maltese"] = "mt";
    languages["Norwegian"] = "no";
    languages["Persian"] = "fa";
    languages["Polish"] = "pl";
    languages["Portuguese"] = "pt";
    languages["Romanian"] = "ro";
    languages["Russian"] = "ru";
    languages["Serbian"] = "sr";
    languages["Slovak"] = "sk";
    languages["Slovenian"] = "sl";
    languages["Spanish"] = "es";
    languages["Swahili"] = "sw";
    languages["Swedish"] = "sv";
    languages["Thai"] = "th";
    languages["Turkish"] = "tr";
    languages["Ukrainian"] = "uk";
    languages["Urdu"] = "ur";
    languages["Vietnamese"] = "vi";
    languages["Welsh"] = "cy";
    languages["Yiddish"] = "yi";
    languages["Detect langlage"] = "";
//    languages["AMHARIC"] = "am";
//    languages["BENGALI"] = "bn";
//    languages["BIHARI"] = "bh";
//    languages["BRETON"] = "br";
//    languages["BURMESE"] = "my";
//    languages["CHEROKEE"] = "chr";
//    languages["CHINESE_SIMPLIFIED"] = "zh-CN";
//    languages["CHINESE_TRADITIONAL"] = "zh-TW";
//    languages["CORSICAN"] = "co";
//    languages["DHIVEHI"] = "dv";
//    languages["ESPERANTO"] = "eo";
//    languages["FAROESE"] = "fo";
//    languages["FRISIAN"] = "fy";
//    languages["GUJARATI"] = "gu";
//    languages["INUKTITUT"] = "iu";
//    languages["JAVANESE"] = "jw";
//    languages["KANNADA"] = "kn";
//    languages["KAZAKH"] = "kk";
//    languages["KHMER"] = "km";
//    languages["KURDISH"] = "ku";
//    languages["KYRGYZ"] = "ky";
//    languages["LAO"] = "lo";
//    languages["LATIN"] = "la";
//    languages["LUXEMBOURGISH"] = "lb";
//    languages["MALAYALAM"] = "ml";
//    languages["MAORI"] = "mi";
//    languages["MARATHI"] = "mr";
//    languages["MONGOLIAN"] = "mn";
//    languages["NEPALI"] = "ne";
//    languages["OCCITAN"] = "oc";
//    languages["ORIYA"] = "or";
//    languages["PASHTO"] = "ps";
//    languages["PORTUGUESE_PORTUGAL"] = "pt-PT";
//    languages["PUNJABI"] = "pa";
//    languages["QUECHUA"] = "qu";
//    languages["SANSKRIT"] = "sa";
//    languages["SCOTS_GAELIC"] = "gd";
//    languages["SINDHI"] = "sd";
//    languages["SINHALESE"] = "si";
//    languages["SUNDANESE"] = "su";
//    languages["SYRIAC"] = "syr";
//    languages["TAJIK"] = "tg";
//    languages["TAMIL"] = "ta";
//    languages["TATAR"] = "tt";
//    languages["TELUGU"] = "te";
//    languages["TIBETAN"] = "bo";
//    languages["TONGA"] = "to";
//    languages["UZBEK"] = "uz";
//    languages["UIGHUR"] = "ug";
}


void GooglePlugin::getDictionaryInfo() {
    QString fullLangFrom=languages.key(_settings->value("lang_from"));
    QString fullLangTo=languages.key(_settings->value("lang_to"));
    _infoNote=" [" + fullLangFrom + "-" + fullLangTo + "] ";
}


Q_EXPORT_PLUGIN2(google, GooglePlugin)
