/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file GooglePlugin.h
    \brief Implementation of google plugin's main class.

    \author Jakub Jaszczynski <j.j.jaszczynski@gmail.com>
*/

#ifndef GOOGLEPLUGIN_H
#define GOOGLEPLUGIN_H


#include <QObject>
#include <QDialog>
#include <QtPlugin>
#include <QIcon>
#include <QtNetwork>

#include "../../include/CommonDictInterface.h"
#include "../../include/settings.h"
#include "../../include/DictDialog.h"
#include "TranslationGoogle.h"
#include "GoogleDictDialog.h"

class GoogleDictDialog;

class GooglePlugin : public CommonDictInterface
{
    Q_OBJECT
    Q_INTERFACES(CommonDictInterface)
public:
    GooglePlugin(QObject *parent=0);
    ~GooglePlugin();

    //! \returns source language code iso 639-2
    QString langFrom() const;

    //! \returns destination language code iso 639-2
    QString langTo() const;

    //! \returns dictionary name (like "old English" or so)
    QString name() const;

    //! \returns dictionary type (xdxf, google translate, etc)
    QString type() const;

    //! returns information about dictionary (name, authors, etc)
    QString infoNote() const;

    //! sets the language to which the translation is done
    void setLangTo(QString langTo);

    //! sets the language from which the translation is done
    void setLangFrom(QString langFrom);

    /*!
        \returns DictDialog object that creates dialogs
        for adding new dictionaries and changing plugin things
    */
    DictDialog* dictDialog();

    //! \returns new, clean copy of plugin with settings set as in Settings*
    CommonDictInterface* getNew(const Settings*) const;

    //! \returns whether plugin can start searching
    bool isAvailable() const;

    //! sets if connection with Internet is possible
    void setConnectionAccept(QString connectionAcepted);

    //! \returns the value of "connection_accepted" from settings
    bool isConnectionAccept() const;

    //! \returns a description of a word given by a QString
    QString search(QString key);

    //! \returns current plugin settings
    Settings* settings();

    //! Sets new settings
    void setSettings(const Settings*);

    //! \returns plugin icon
    QIcon* icon();

    //! \returns empty translation object (to be fetched later) for a given key
    Translation* getTranslationFor(QString key);

    //! initializes the list of available languages in Google translator
    static void initLanguages();

    static QMap<QString, QString> languages;

public slots:
    /*! performs search in a dictionary
      \param  word word to search for in a dictionary
      \param limit limit on number of results

      After finishing search it has to emit
      \see CommonDictInterface:finalTranslation  finalTranslation
    */
    QList<Translation*> searchWordList(QString word, int limit=0);

    //! stops current operation
    void stop();

    //! function called after the request from Google is returned
    void done();

    //! transforms Google format to String with translation
    QString jsonParse(QString result);

    //! sets information about dictionary
    void getDictionaryInfo();

    //! loads translations for each plugin only once
    void retranslate();
protected:
    static bool noNetworkErrorShowed;

private:
    //! name of a dictionary
    QString _name;
    //! type of a dictionary
    QString _type;
    //! information about dictionary
    QString _infoNote;

    //! icon displayed during translations and when a dictionary is chosen
    QIcon _icon;
    Settings *_settings;
    //! indicates if search is stopped
    bool stopped;
    bool _connectionAccept;
    //! indicates if response from Google appeared
    volatile bool wait;
    QHttp *http;
    GoogleDictDialog *_dictDialog;
    QThread *threa;
};

#endif // GOOGLEPLUGIN_H


